<?php

// Exit if accessed directly
if ( !defined( 'ABSPATH' ) ) exit;

// Check if class already exists

//if (!class_exists("WPsite_Stats_FB")):

// AJAX functions for impression and click tracking

add_action( 'wp_ajax_wps_fb_record_impression', array('WPsite_Stats_FB', 'recordImpresssion'));
add_action( 'wp_ajax_nopriv_wps_fb_record_impression', array('WPsite_Stats_FB', 'recordImpresssion'));
add_action( 'wp_ajax_wps_fb_record_click', array('WPsite_Stats_FB', 'recordClick'));
add_action( 'wp_ajax_nopriv_wps_fb_record_click', array('WPsite_Stats_FB', 'recordClick'));

/**
 * WPsite_Stats_FB class.
 *
 * This class is used to record all stats for firebars.
 */
class WPsite_Stats_FB {

	/**
	 * table_name
	 *
	 * (default value: 'wpsite_firebar_stats')
	 *
	 * @var string
	 * @access public
	 * @static
	 */
	public static $table_name = 'wpsite_firebar_stats';

	/**
	 * Create Table when this class is loaded
	 *
	 * @access public
	 * @return void
	 */
	function __construct() {
		self::createTable();
	}

	/**
	 * Creates the Stats table if it does not exist
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function createTable(){

		global $wpdb;

		$wpdb->query("
			CREATE TABLE IF NOT EXISTS `" . self::getTableName() . "` (
				`id` int(11) NOT NULL AUTO_INCREMENT,
				`date` date NOT NULL,
				`firebar_id` int(11) NOT NULL DEFAULT 0,
				`impressions` int(8) NOT NULL DEFAULT 0,
				`clicks` int(8) NOT NULL DEFAULT 0,
				PRIMARY KEY (`id`)
			) ENGINE=InnoDB DEFAULT CHARSET=latin1 AUTO_INCREMENT=1;
		");

	}

	/**
	 * Get the stats for all firebars in a timeframe or a single firbar in a timeframe
	 *
	 * @access public
	 * @static
	 * @param mixed $start
	 * @param mixed $end
	 * @param mixed $id (default: null)
	 * @return void
	 */
	static function getStats($start, $end, $id = null, $select = '*') {

		$query = null;

		global $wpdb;

		// All Firebars, All Time

		if ($start == null && $end == null && $id == null) {
			$query = 'SELECT ' . $select . ' FROM ' . self::getTableName();
		}

		// All Firebars, After Date

		else if ($start != null && $end == null && $id == null) {
			$query = $wpdb->prepare('SELECT ' . $select . ' FROM ' . self::getTableName() . ' WHERE `date` = %s', $start);
		}

		// All Firebars, Date Range

		else if ($start != null && $end != null && $id == null) {
			$query = $wpdb->prepare('SELECT ' . $select . ' FROM ' . self::getTableName() . ' WHERE `date` >= %s AND `date` <= %s', $start, $end);
		}

		// Single Firebar, All Time

		else if ($start == null && $end == null && $id != null) {
			$query = $wpdb->prepare('SELECT ' . $select . ' FROM ' . self::getTableName() . ' WHERE `firebar_id` = %d', $id);
		}

		// Single Firebar, Date Range

		else if ($start != null && $end != null && $id != null) {
			$query = $wpdb->prepare('SELECT ' . $select . ' FROM ' . self::getTableName() . ' WHERE `date` >= %s AND `date` <= %s AND `firebar_id` = %d', $start, $end, $id);
		}

		// No query was created

		if (!isset($query)) {
			return false;
		}

		$result = $wpdb->get_results($query, 'ARRAY_A');

		return $result;
	}

	/**
	 * Runs everyday and will add a record to table if none exists
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function initialize_firebar_stats() {

		include_once(WPSITE_FIREBAR_PLUGIN_DIR . 'admin/model/db.php');

		global $wpdb;
		$today = date(WPSITE_FIREBAR_DATE_FORMAT);

		$firebars = $wpdb->get_results("SELECT * FROM `" . WPsite_Firebar_DB::getTableName() . "`");

		$result = false;

		foreach ($firebars as $firebar) {

			// Check if entry already exsits

			$query = $wpdb->prepare('SELECT * FROM ' . self::getTableName() . ' WHERE `date` = %s AND `firebar_id` = %d', $today, $firebar->id);
			$firebar_impressions = $wpdb->query($query);

			// Entry does not exist, create a new one and set impressions to 0 and clicks to 0

			if (!isset($firebar_impressions) || $firebar_impressions == 0) {

				$query = $wpdb->prepare('INSERT INTO ' . self::getTableName() . ' (
					`date`,
					`firebar_id`,
					`impressions`,
					`clicks`
					) VALUES (%s, %d, 0, 0)',
					$today,
					$firebar->id
				);

				$result = $wpdb->query($query);
			}
		}

		return $result;

	}

	/**
	 * Record an impression made by a firebar
	 * This function is called via PHP.
	 *
	 * @access public
	 * @static
	 * @param mixed $data (default: null)
	 * @return void
	 */
	static function recordImpresssion($data = null) {

		$data['firebar_id'] = $_POST['firebar_id'];

		// Return false if no data is given

		if (!isset($data) || !isset($data['firebar_id'])) {
			echo 'Firebar impression was NOT able to be recored because the Firebar ID was not retrieved.';
			die();
		}

		// Return false if crawler

		if (self::isBot()) {
			echo 'Firebar impression was NOT able to be recored because a crawler was detected.';
			die();
		}

		// Return if user is admin or higher

		if (current_user_can('edit_published_posts')) {
			echo 'Firebar impression was NOT able to be recored because current user is an admin';
			die();
		}

		global $wpdb;
		$today = date(WPSITE_FIREBAR_DATE_FORMAT);

		// Check if entry already exsits

		$query = $wpdb->prepare('SELECT * FROM ' . self::getTableName() . ' WHERE `date` = %s AND `firebar_id` = %d', $today, $data['firebar_id']);
		$firebar_impressions = $wpdb->query($query);

		// Entry aleady exists, just add 1

		if (isset($firebar_impressions) && $firebar_impressions != 0) {

			$query = $wpdb->prepare('UPDATE ' . self::getTableName() . ' SET
				`impressions` = `impressions` + 1
				WHERE `date` = %s AND `firebar_id` = %d', $today, $data['firebar_id']);

			$result = $wpdb->query($query);

		}

		// Entry does not exist, create a new one and set impressions to 1

		else {
			$query = $wpdb->prepare('INSERT INTO ' . self::getTableName() . ' (
				`date`,
				`firebar_id`,
				`impressions`,
				`clicks`
				) VALUES (%s, %d, 1, 0)',
				$today,
				$data['firebar_id']
			);

			$result = $wpdb->query($query);
		}

		if ($result) {
			echo 'Firebar impression was able to be recored.';
		} else {
			echo 'Firebar click was NOT able to be recored.';
		}

		die(); // necessary for WordPress AJAX calls

	}

	/**
	 * Record a click made by a firebar
	 * This function is called via AJAX
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function recordClick() {

		$firebar_id = $_POST['firebar_id'];

		// Return if no data is given

		if (!isset($firebar_id)) {
			echo 'Firebar click was NOT able to be recored because the Firebar ID was not retrieved.';
			die();
		}

		// Return if user is admin or higher

		if (current_user_can('edit_published_posts')) {
			echo 'Firebar click was NOT able to be recored because current user is not authorized to log clicks.';
			die();
		}

		// Return false if crawler

		if (self::isBot()) {
			echo 'Firebar click was NOT able to be recored because there is no user.  A crawler was detected.';
			die();
		}

		global $wpdb;
		$today = date(WPSITE_FIREBAR_DATE_FORMAT);

		// Check if entry already exsits

		$query = $wpdb->prepare('SELECT * FROM ' . self::getTableName() . ' WHERE `date` = %s AND `firebar_id` = %d', $today, $firebar_id);
		$firebar_clicks = $wpdb->query($query);

		// Entry aleady exists, just add 1

		if (isset($firebar_clicks) && $firebar_clicks != 0) {

			$query = $wpdb->prepare('UPDATE ' . self::getTableName() . ' SET
				`clicks` = `clicks` + 1
				WHERE `date` = %s AND `firebar_id` = %d', $today, $firebar_id);

			$result = $wpdb->query($query);

		}

		// Entry does not exist, create a new one and set impressions to 1

		else {
			$query = $wpdb->prepare('INSERT INTO ' . self::getTableName() . ' (
				`date`,
				`firebar_id`,
				`impressions`,
				`clicks`
				) VALUES (%s, %d, 0, 1)',
				$today,
				$firebar_id
			);

			$result = $wpdb->query($query);
		}

		if ($result) {
			echo 'Firebar click was able to be recored.';
		} else {
			echo 'Firebar click was NOT able to be recored.';
		}

		die(); // necessary for WordPress AJAX calls
	}

	/**
	 * Deletes the stats for a firebar when the firebar is deleted
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function deleteStats($firebar_id) {

		// Return false if firebar_id is not set

		if (!isset($firebar_id)) {
			return false;
		}

		global $wpdb;

		// Check if entry already exsits

		$query = $wpdb->prepare('DELETE FROM ' . self::getTableName() . ' WHERE `firebar_id` = %d', $firebar_id);
		$result = $wpdb->query($query);

		return $result;

	}

	/**
	 * Check for BOTs visiting site
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function isBot() {

		if (isset($_SERVER['HTTP_USER_AGENT']) && preg_match('/bot|spider|crawler|curl|^$/i', $_SERVER['HTTP_USER_AGENT'])) {
			return true;
		} else {
			return false;
		}

	}

	/**
	 * Returns the proper table name based off of site ID
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function getTableName(){

		global $wpdb;

		return $wpdb->prefix . self::$table_name;
	}

}

//endif;